@@ -60,6 +60,7 @@ import Data.Monoid (Monoid (mappend, mempty))
 import Control.Monad.Trans.Resource
 import qualified GHC.Exts
 import qualified Data.IORef as I
+import Control.Monad.Morph
 
 -- | The underlying datatype for all the types in this package.  In has six
 -- type parameters:
@@ -111,9 +112,9 @@ instance Monad m => Applicative (Pipe l i o u m) where
 instance Monad m => Monad (Pipe l i o u m) where
     return = Done
 
-    Done x           >>= fp = fp x
     HaveOutput p c o >>= fp = HaveOutput (p >>= fp)            c          o
     NeedInput p c    >>= fp = NeedInput  (p >=> fp)            (c >=> fp)
+    Done x           >>= fp = fp x
     PipeM mp         >>= fp = PipeM      ((>>= fp) `liftM` mp)
     Leftover p i     >>= fp = Leftover   (p >>= fp)            i
 
@@ -139,8 +140,15 @@ instance Monad m => Monoid (Pipe l i o u m ()) where
 instance MonadResource m => MonadResource (Pipe l i o u m) where
     liftResourceT = lift . liftResourceT
 
+-- | Core datatype of the conduit package. This type represents a general
+-- component which can consume a stream of input values @i@, produce a stream
+-- of output values @o@, perform actions in the @m@ monad, and produce a final
+-- result @r@. The type synonyms provided here are simply wrappers around this
+-- type.
+--
+-- Since 1.0.0
 newtype ConduitM i o m r = ConduitM { unConduitM :: Pipe i i o () m r }
-    deriving (Functor, Applicative, Monad, MonadIO, MonadTrans, MonadThrow, MonadActive, MonadResource)
+    deriving (Functor, Applicative, Monad, MonadIO, MonadTrans, MonadThrow, MonadActive, MonadResource, MFunctor)
 instance MonadBase base m => MonadBase base (ConduitM i o m) where
     liftBase = lift . liftBase
 instance Monad m => Monoid (ConduitM i o m ()) where
@@ -153,9 +161,11 @@ instance Monad m => Monoid (ConduitM i o m ()) where
 -- Since 0.5.0
 type Source m o = ConduitM () o m ()
 
--- | Generalized 'Source'.
+-- | A component which produces a stream of output values, regardless of the
+-- input stream. A @Producer@ is a generalization of a @Source@, and can be
+-- used as either a @Source@ or a @Conduit@.
 --
--- Since 0.5.0
+-- Since 1.0.0
 type Producer m o = forall i. ConduitM i o m ()
 
 -- | Consumes a stream of input values and produces a final result, without
@@ -164,9 +174,11 @@ type Producer m o = forall i. ConduitM i o m ()
 -- Since 0.5.0
 type Sink i m r = ConduitM i Void m r
 
--- | Generalized 'Sink' without leftovers.
+-- | A component which consumes a stream of input values and produces a final
+-- result, regardless of the output stream. A @Consumer@ is a generalization of
+-- a @Sink@, and can be used as either a @Sink@ or a @Conduit@.
 --
--- Since 0.5.0
+-- Since 1.0.0
 type Consumer i m r = forall o. ConduitM i o m r
 
 -- | Consumes a stream of input values and produces a stream of output values,
@@ -183,8 +195,7 @@ type Conduit i m o = ConduitM i o m ()
 -- Since 0.5.0
 data ResumableSource m o = ResumableSource (Source m o) (m ())
 
--- | Wait for a single input value from upstream, terminating immediately if no
--- data is available.
+-- | Wait for a single input value from upstream.
 --
 -- Since 0.5.0
 await :: Pipe l i o u m (Maybe i)
@@ -302,25 +313,27 @@ idP = NeedInput (HaveOutput idP (return ())) Done
 -- Since 0.5.0
 pipe :: Monad m => Pipe l a b r0 m r1 -> Pipe Void b c r1 m r2 -> Pipe l a c r0 m r2
 pipe =
-    pipe' (return ())
+    goRight (return ())
   where
-    pipe' final left right =
+    goRight final left right =
         case right of
-            Done r2 -> PipeM (final >> return (Done r2))
-            HaveOutput p c o -> HaveOutput (pipe' final left p) (c >> final) o
-            PipeM mp -> PipeM (liftM (pipe' final left) mp)
-            Leftover _ i -> absurd i
-            NeedInput rp rc -> upstream rp rc
+            HaveOutput p c o -> HaveOutput (recurse p) (c >> final) o
+            NeedInput rp rc  -> goLeft rp rc final left
+            Done r2          -> PipeM (final >> return (Done r2))
+            PipeM mp         -> PipeM (liftM recurse mp)
+            Leftover _ i     -> absurd i
       where
-        upstream rp rc =
-            case left of
-                Done r1 -> pipe (Done r1) (rc r1)
-                HaveOutput left' final' o -> pipe' final' left' (rp o)
-                PipeM mp -> PipeM (liftM (\left' -> pipe' final left' right) mp)
-                Leftover left' i -> Leftover (pipe' final left' right) i
-                NeedInput left' lc -> NeedInput
-                    (\a -> pipe' final (left' a) right)
-                    (\r0 -> pipe' final (lc r0) right)
+        recurse = goRight final left
+
+    goLeft rp rc final left =
+        case left of
+            HaveOutput left' final' o -> goRight final' left' (rp o)
+            NeedInput left' lc        -> NeedInput (recurse . left') (recurse . lc)
+            Done r1                   -> goRight (return ()) (Done r1) (rc r1)
+            PipeM mp                  -> PipeM (liftM recurse mp)
+            Leftover left' i          -> Leftover (recurse left') i
+      where
+        recurse = goLeft rp rc final
 
 -- | Same as 'pipe', but automatically applies 'injectLeftovers' to the right @Pipe@.
 --
@@ -332,24 +345,27 @@ pipeL :: Monad m => Pipe l a b r0 m r1 -> Pipe b b c r1 m r2 -> Pipe l a c r0 m
 --
 -- However, this version tested as being significantly more efficient.
 pipeL =
-    pipe' (return ())
+    goRight (return ())
   where
-    pipe' :: Monad m => m () -> Pipe l a b r0 m r1 -> Pipe b b c r1 m r2 -> Pipe l a c r0 m r2
-    pipe' final left right =
+    goRight final left right =
         case right of
-            Done r2 -> PipeM (final >> return (Done r2))
-            HaveOutput p c o -> HaveOutput (pipe' final left p) (c >> final) o
-            PipeM mp -> PipeM (liftM (pipe' final left) mp)
-            Leftover right' i -> pipe' final (HaveOutput left final i) right'
-            NeedInput rp rc ->
-                case left of
-                    Done r1 -> pipe' (return ()) (Done r1) (rc r1)
-                    HaveOutput left' final' o -> pipe' final' left' (rp o)
-                    PipeM mp -> PipeM (liftM (\left' -> pipe' final left' right) mp)
-                    NeedInput left' lc -> NeedInput
-                        (\a -> pipe' final (left' a) right)
-                        (\r0 -> pipe' final (lc r0) right)
-                    Leftover left' i -> Leftover (pipe' final left' right) i
+            HaveOutput p c o  -> HaveOutput (recurse p) (c >> final) o
+            NeedInput rp rc   -> goLeft rp rc final left
+            Done r2           -> PipeM (final >> return (Done r2))
+            PipeM mp          -> PipeM (liftM recurse mp)
+            Leftover right' i -> goRight final (HaveOutput left final i) right'
+      where
+        recurse = goRight final left
+
+    goLeft rp rc final left =
+        case left of
+            HaveOutput left' final' o -> goRight final' left' (rp o)
+            NeedInput left' lc        -> NeedInput (recurse . left') (recurse . lc)
+            Done r1                   -> goRight (return ()) (Done r1) (rc r1)
+            PipeM mp                  -> PipeM (liftM recurse mp)
+            Leftover left' i          -> Leftover (recurse left') i
+      where
+        recurse = goLeft rp rc final
 
 -- | Connect a @Source@ to a @Sink@ until the latter closes. Returns both the
 -- most recent state of the @Source@ and the result of the @Sink@.
@@ -362,22 +378,26 @@ connectResume :: Monad m
               => ResumableSource m o
               -> Sink o m r
               -> m (ResumableSource m o, r)
-connectResume (ResumableSource (ConduitM left0) leftFinal0) =
-    go leftFinal0 left0 . unConduitM
+connectResume (ResumableSource (ConduitM left0) leftFinal0) (ConduitM right0) =
+    goRight leftFinal0 left0 right0
   where
-    go leftFinal left right =
+    goRight leftFinal left right =
         case right of
-            Done r2 -> return (ResumableSource (ConduitM left) leftFinal, r2)
-            PipeM mp -> mp >>= go leftFinal left
             HaveOutput _ _ o -> absurd o
-            Leftover p i -> go leftFinal (HaveOutput left leftFinal i) p
-            NeedInput rp rc ->
-                case left of
-                    Leftover p () -> go leftFinal p right
-                    HaveOutput left' leftFinal' o -> go leftFinal' left' (rp o)
-                    NeedInput _ lc -> go leftFinal (lc ()) right
-                    Done () -> go (return ()) (Done ()) (rc ())
-                    PipeM mp -> mp >>= \left' -> go leftFinal left' right
+            NeedInput rp rc  -> goLeft rp rc leftFinal left
+            Done r2          -> return (ResumableSource (ConduitM left) leftFinal, r2)
+            PipeM mp         -> mp >>= goRight leftFinal left
+            Leftover p i     -> goRight leftFinal (HaveOutput left leftFinal i) p
+
+    goLeft rp rc leftFinal left =
+        case left of
+            HaveOutput left' leftFinal' o -> goRight leftFinal' left' (rp o)
+            NeedInput _ lc                -> recurse (lc ())
+            Done ()                       -> goRight (return ()) (Done ()) (rc ())
+            PipeM mp                      -> mp >>= recurse
+            Leftover p ()                 -> recurse p
+      where
+        recurse = goLeft rp rc leftFinal
 
 -- | Run a pipeline until processing completes.
 --
@@ -401,12 +421,12 @@ injectLeftovers :: Monad m => Pipe i i o u m r -> Pipe l i o u m r
 injectLeftovers =
     go []
   where
-    go _ (Done r) = Done r
     go ls (HaveOutput p c o) = HaveOutput (go ls p) c o
-    go ls (PipeM mp) = PipeM (liftM (go ls) mp)
-    go ls (Leftover p l) = go (l:ls) p
     go (l:ls) (NeedInput p _) = go ls $ p l
     go [] (NeedInput p c) = NeedInput (go [] . p) (go [] . c)
+    go _ (Done r) = Done r
+    go ls (PipeM mp) = PipeM (liftM (go ls) mp)
+    go ls (Leftover p l) = go (l:ls) p
 
 -- | Transform the monad that a @Pipe@ lives in.
 --
@@ -416,6 +436,8 @@ injectLeftovers =
 --
 -- <https://github.com/snoyberg/conduit/wiki/Dealing-with-monad-transformers>
 --
+-- This function is just a synonym for 'hoist'.
+--
 -- Since 0.4.0
 transPipe :: Monad m => (forall a. m a -> n a) -> Pipe l i o u m r -> Pipe l i o u n r
 transPipe f (HaveOutput p c o) = HaveOutput (transPipe f p) (f c) o
@@ -499,30 +521,30 @@ sourceToPipe :: Monad m => Source m o -> Pipe l i o u m ()
 sourceToPipe =
     go . unConduitM
   where
+    go (HaveOutput p c o) = HaveOutput (go p) c o
+    go (NeedInput _ c) = go $ c ()
     go (Done ()) = Done ()
     go (PipeM mp) = PipeM (liftM go mp)
-    go (NeedInput _ c) = go $ c ()
-    go (HaveOutput p c o) = HaveOutput (go p) c o
     go (Leftover p ()) = go p
 
 sinkToPipe :: Monad m => Sink i m r -> Pipe l i o u m r
 sinkToPipe =
     go . injectLeftovers . unConduitM
   where
+    go (HaveOutput _ _ o) = absurd o
+    go (NeedInput p c) = NeedInput (go . p) (const $ go $ c ())
     go (Done r) = Done r
     go (PipeM mp) = PipeM (liftM go mp)
-    go (NeedInput p c) = NeedInput (go . p) (const $ go $ c ())
-    go (HaveOutput _ _ o) = absurd o
     go (Leftover _ l) = absurd l
 
 conduitToPipe :: Monad m => Conduit i m o -> Pipe l i o u m ()
 conduitToPipe =
     go . injectLeftovers . unConduitM
   where
+    go (HaveOutput p c o) = HaveOutput (go p) c o
+    go (NeedInput p c) = NeedInput (go . p) (const $ go $ c ())
     go (Done ()) = Done ()
     go (PipeM mp) = PipeM (liftM go mp)
-    go (NeedInput p c) = NeedInput (go . p) (const $ go $ c ())
-    go (HaveOutput p c o) = HaveOutput (go p) c o
     go (Leftover _ l) = absurd l
 
 -- | Returns a tuple of the upstream and downstream results. Note that this
@@ -548,6 +570,7 @@ withUpstream down =
 -- appropriately. By appropriately, I mean:
 --
 -- * If a new finalizer is registered, the old one should not be called.
+--
 -- * If the old one is called, it should not be called again.
 --
 -- This function returns both a @Source@ and a finalizer which ensures that the
@@ -570,14 +593,7 @@ infixl 9 >+>
 --
 -- Notice that the /leftover/ parameter for the @Pipe@s must be @Void@. This
 -- ensures that there is no accidental data loss of leftovers during fusion. If
--- you have a @Pipe@ with leftovers, you must first call 'injectLeftovers'. For
--- example:
---
--- > :load Data.Conduit.List
--- > :set -XNoMonomorphismRestriction
--- > let pipe = peek >>= \x -> fold (Prelude.+) 0 >>= \y -> return (x, y)
--- > runPipe $ sourceList [1..10] >+> injectLeftovers pipe
--- (Just 1,55)
+-- you have a @Pipe@ with leftovers, you must first call 'injectLeftovers'.
 --
 -- Since 0.5.0
 (>+>) :: Monad m => Pipe l a b r0 m r1 -> Pipe Void b c r1 m r2 -> Pipe l a c r0 m r2
@@ -591,7 +607,7 @@ infixl 9 >+>
 (<+<) = flip pipe
 {-# INLINE (<+<) #-}
 
--- | Convert a 'Source' into a 'Producer'.
+-- | Generalize a 'Source' to a 'Producer'.
 --
 -- Since 1.0.0
 toProducer :: Monad m => Source m a -> Producer m a
@@ -604,7 +620,7 @@ toProducer =
     go (PipeM mp) = PipeM (liftM go mp)
     go (Leftover p ()) = go p
 
--- | Convert a 'Sink' into a 'Consumer'.
+-- | Generalize a 'Sink' to a 'Consumer'.
 --
 -- Since 1.0.0
 toConsumer :: Monad m => Sink a m b -> Consumer a m b
@@ -616,3 +632,7 @@ toConsumer =
     go (Done r) = Done r
     go (PipeM mp) = PipeM (liftM go mp)
     go (Leftover p l) = Leftover (go p) l
+
+-- | Since 1.0.4
+instance MFunctor (Pipe l i o u) where
+    hoist = transPipe
