@@ -62,8 +62,7 @@ import Prelude
     , either
     )
 import Data.Monoid (Monoid, mempty, mappend)
-import Data.Conduit hiding (Source, Sink, Conduit, Pipe)
-import Data.Conduit.Internal (sourceList, pipeL, pipe)
+import Data.Conduit
 import Control.Monad (when, (<=<))
 import Control.Monad.Trans.Class (lift)
 
@@ -73,7 +72,7 @@ import Control.Monad.Trans.Class (lift)
 unfold :: Monad m
        => (b -> Maybe (a, b))
        -> b
-       -> GSource m a
+       -> Producer m a
 unfold f =
     go
   where
@@ -82,6 +81,9 @@ unfold f =
             Just (a, seed') -> yield a >> go seed'
             Nothing -> return ()
 
+sourceList :: Monad m => [a] -> Producer m a
+sourceList = Prelude.mapM_ yield
+
 -- | Enumerate from a value to a final value, inclusive, via 'succ'.
 --
 -- This is generally more efficient than using @Prelude@\'s @enumFromTo@ and
@@ -92,7 +94,7 @@ unfold f =
 enumFromTo :: (Enum a, Eq a, Monad m)
            => a
            -> a
-           -> GSource m a
+           -> Producer m a
 enumFromTo start stop =
     go start
   where
@@ -101,7 +103,7 @@ enumFromTo start stop =
         | otherwise = yield i >> go (succ i)
 
 -- | Produces an infinite stream of repeated applications of f to x.
-iterate :: Monad m => (a -> a) -> a -> GSource m a
+iterate :: Monad m => (a -> a) -> a -> Producer m a
 iterate f =
     go
   where
@@ -113,7 +115,7 @@ iterate f =
 fold :: Monad m
      => (b -> a -> b)
      -> b
-     -> GSink a m b
+     -> Consumer a m b
 fold f =
     loop
   where
@@ -130,7 +132,7 @@ fold f =
 foldM :: Monad m
       => (b -> a -> m b)
       -> b
-      -> GSink a m b
+      -> Consumer a m b
 foldM f =
     loop
   where
@@ -146,7 +148,7 @@ foldM f =
 -- Since 0.5.3
 foldMap :: (Monad m, Monoid b)
         => (a -> b)
-        -> GSink a m b
+        -> Consumer a m b
 foldMap f =
     fold combiner mempty
   where
@@ -157,7 +159,7 @@ foldMap f =
 -- Since 0.3.0
 mapM_ :: Monad m
       => (a -> m ())
-      -> GInfSink a m
+      -> Consumer a m ()
 mapM_ f = awaitForever $ lift . f
 
 -- | Ignore a certain number of values in the stream. This function is
@@ -171,7 +173,7 @@ mapM_ f = awaitForever $ lift . f
 -- Since 0.3.0
 drop :: Monad m
      => Int
-     -> GSink a m ()
+     -> Consumer a m ()
 drop =
     loop
   where
@@ -187,7 +189,7 @@ drop =
 -- Since 0.3.0
 take :: Monad m
      => Int
-     -> GSink a m [a]
+     -> Consumer a m [a]
 take =
     loop id
   where
@@ -199,20 +201,20 @@ take =
 -- | Take a single value from the stream, if available.
 --
 -- Since 0.3.0
-head :: Monad m => GSink a m (Maybe a)
+head :: Monad m => Consumer a m (Maybe a)
 head = await
 
 -- | Look at the next value in the stream, if available. This function will not
 -- change the state of the stream.
 --
 -- Since 0.3.0
-peek :: Monad m => GLSink a m (Maybe a)
+peek :: Monad m => Consumer a m (Maybe a)
 peek = await >>= maybe (return Nothing) (\x -> leftover x >> return (Just x))
 
 -- | Apply a transformation to all values in a stream.
 --
 -- Since 0.3.0
-map :: Monad m => (a -> b) -> GInfConduit a m b
+map :: Monad m => (a -> b) -> Conduit a m b
 map f = awaitForever $ yield . f
 
 {-
@@ -243,7 +245,7 @@ differences based on leftovers.
 -- side-effects of running the action, see 'mapM_'.
 --
 -- Since 0.3.0
-mapM :: Monad m => (a -> m b) -> GInfConduit a m b
+mapM :: Monad m => (a -> m b) -> Conduit a m b
 mapM f = awaitForever $ yield <=< lift . f
 
 -- | Apply a monadic action on all values in a stream.
@@ -254,52 +256,52 @@ mapM f = awaitForever $ yield <=< lift . f
 -- > iterM f = mapM (\a -> f a >>= \() -> return a)
 --
 -- Since 0.5.6
-iterM :: Monad m => (a -> m ()) -> GInfConduit a m a
+iterM :: Monad m => (a -> m ()) -> Conduit a m a
 iterM f = awaitForever $ \a -> lift (f a) >> yield a
 
 -- | Apply a transformation that may fail to all values in a stream, discarding
 -- the failures.
 --
 -- Since 0.5.1
-mapMaybe :: Monad m => (a -> Maybe b) -> GInfConduit a m b
+mapMaybe :: Monad m => (a -> Maybe b) -> Conduit a m b
 mapMaybe f = awaitForever $ maybe (return ()) yield . f
 
 -- | Apply a monadic transformation that may fail to all values in a stream,
 -- discarding the failures.
 --
 -- Since 0.5.1
-mapMaybeM :: Monad m => (a -> m (Maybe b)) -> GInfConduit a m b
+mapMaybeM :: Monad m => (a -> m (Maybe b)) -> Conduit a m b
 mapMaybeM f = awaitForever $ maybe (return ()) yield <=< lift . f
 
 -- | Filter the @Just@ values from a stream, discarding the @Nothing@  values.
 --
 -- Since 0.5.1
-catMaybes :: Monad m => GInfConduit (Maybe a) m a
+catMaybes :: Monad m => Conduit (Maybe a) m a
 catMaybes = awaitForever $ maybe (return ()) yield
 
 -- | Apply a transformation to all values in a stream, concatenating the output
 -- values.
 --
 -- Since 0.3.0
-concatMap :: Monad m => (a -> [b]) -> GInfConduit a m b
+concatMap :: Monad m => (a -> [b]) -> Conduit a m b
 concatMap f = awaitForever $ sourceList . f
 
 -- | Apply a monadic transformation to all values in a stream, concatenating
 -- the output values.
 --
 -- Since 0.3.0
-concatMapM :: Monad m => (a -> m [b]) -> GInfConduit a m b
+concatMapM :: Monad m => (a -> m [b]) -> Conduit a m b
 concatMapM f = awaitForever $ sourceList <=< lift . f
 
 -- | 'concatMap' with an accumulator.
 --
 -- Since 0.3.0
-concatMapAccum :: Monad m => (a -> accum -> (accum, [b])) -> accum -> GInfConduit a m b
+concatMapAccum :: Monad m => (a -> accum -> (accum, [b])) -> accum -> Conduit a m b
 concatMapAccum f =
     loop
   where
     loop accum =
-        awaitE >>= either return go
+        await >>= maybe (return ()) go
       where
         go a = do
             let (accum', bs) = f a accum
@@ -309,12 +311,12 @@ concatMapAccum f =
 -- | 'concatMapM' with an accumulator.
 --
 -- Since 0.3.0
-concatMapAccumM :: Monad m => (a -> accum -> m (accum, [b])) -> accum -> GInfConduit a m b
+concatMapAccumM :: Monad m => (a -> accum -> m (accum, [b])) -> accum -> Conduit a m b
 concatMapAccumM f =
     loop
   where
     loop accum = do
-        awaitE >>= either return go
+        await >>= maybe (return ()) go
       where
         go a = do
             (accum', bs) <- lift $ f a accum
@@ -326,7 +328,7 @@ concatMapAccumM f =
 -- "Data.Conduit.Lazy".
 --
 -- Since 0.3.0
-consume :: Monad m => GSink a m [a]
+consume :: Monad m => Consumer a m [a]
 consume =
     loop id
   where
@@ -335,14 +337,14 @@ consume =
 -- | Grouping input according to an equality function.
 --
 -- Since 0.3.0
-groupBy :: Monad m => (a -> a -> Bool) -> GInfConduit a m [a]
+groupBy :: Monad m => (a -> a -> Bool) -> Conduit a m [a]
 groupBy f =
     start
   where
-    start = awaitE >>= either return (loop id)
+    start = await >>= maybe (return ()) (loop id)
 
     loop rest x =
-        awaitE >>= either (\r -> yield (x : rest []) >> return r) go
+        await >>= maybe (yield (x : rest [])) go
       where
         go y
             | f x y     = loop (rest . (y:)) x
@@ -361,7 +363,7 @@ groupBy f =
 -- >     ...
 --
 -- Since 0.3.0
-isolate :: Monad m => Int -> GConduit a m a
+isolate :: Monad m => Int -> Conduit a m a
 isolate =
     loop
   where
@@ -371,21 +373,21 @@ isolate =
 -- | Keep only values in the stream passing a given predicate.
 --
 -- Since 0.3.0
-filter :: Monad m => (a -> Bool) -> GInfConduit a m a
+filter :: Monad m => (a -> Bool) -> Conduit a m a
 filter f = awaitForever $ \i -> when (f i) (yield i)
 
 -- | Ignore the remainder of values in the source. Particularly useful when
 -- combined with 'isolate'.
 --
 -- Since 0.3.0
-sinkNull :: Monad m => GInfSink a m
+sinkNull :: Monad m => Consumer a m ()
 sinkNull = awaitForever $ \_ -> return ()
 
 -- | A source that outputs no values. Note that this is just a type-restricted
 -- synonym for 'mempty'.
 --
 -- Since 0.3.0
-sourceNull :: Monad m => GSource m a
+sourceNull :: Monad m => Producer m a
 sourceNull = return ()
 
 -- | Run a @Pipe@ repeatedly, and output its result value downstream. Stops
@@ -393,8 +395,8 @@ sourceNull = return ()
 --
 -- Since 0.5.0
 sequence :: Monad m
-         => GLSink i m o -- ^ @Pipe@ to run repeatedly
-         -> GLInfConduit i m o
+         => Consumer i m o -- ^ @Pipe@ to run repeatedly
+         -> Conduit i m o
 sequence sink =
     self
   where
